"""Telegram callback model"""

from typing import Any, Coroutine, Literal

from cryptography.fernet import Fernet

from ..encryption.mixins import EncryptionMixin
from .callback_models import BaseSchema, HttpCallback
from .types import Int


class TelegramParams(BaseSchema):
    """Callback request parameters"""

    # callback request timeout
    timeout: Int(minValue=0) = 60


class TelegramCallback(HttpCallback, EncryptionMixin):
    """Telegram notification policy callbacks"""

    # callback type
    type: Literal["telegram"]
    # telegram account type
    chatId: int
    # authorization token
    token: str
    # callback request parameters
    params: TelegramParams = TelegramParams()
    # callback url
    url: str = ""

    def getJsonBody(self, data: dict) -> dict | None:
        """
        Description see :func:`~_HttpCallback.getJsonBody`.
        """
        return data

    def model_post_init(self, __context):
        """HttpCallback overridden method"""
        self._headers = {}

    def encryptToken(self) -> None:
        """Encrypt telegram token"""
        key = self.getEncryptionKey(str(self.chatId))
        return Fernet(key).encrypt(self.token.encode()).decode()

    def decryptToken(self) -> None:
        """Decrypt telegram token"""
        key = self.getEncryptionKey(str(self.chatId))
        return Fernet(key).decrypt(self.token.encode()).decode()

    def buildCallback(self, data: Any) -> Coroutine | None:
        """
        Build callback coroutine
        Args:
            data: data to send

        Returns:
            callback coroutine or None if callback disabled or filtered
        """
        self.url = f"https://api.telegram.org/bot{self.decryptToken()}/sendMessage?chat_id={self.chatId}"
        return super().buildCallback({"text": str(data)})

    def asDict(self) -> dict:
        """Telegram callback as dict"""
        return {
            "type": self.type,
            "chat_id": self.chatId,
            "token": self.encryptToken(),
            "params": self.params.asDict(),
            "enable": self.enable,
        }
